/*
	Serial
*/

//-----------------------------------------------------------------------------
// USART Fuction Declaration


//#ifdef _UART0_ENABLE
//------------------------------------------------------------------------------
// UART Defination
//------------------------------------------------------------------------------
#define U0_printf							printf

#define BAUD_9600							0
#define BAUD_14400							1
#define BAUD_19200							2
#define BAUD_28800							3
#define BAUD_38400							4		
#define BAUD_57600							5		
#define BAUD_76800							6		
#define BAUD_115200							7	

#ifdef _SYS_CLK_AT16MHZ
#define BAUDRATE_VAL_9600					207
#define BAUDRATE_VAL_19200					103
#define BAUDRATE_VAL_14400					138
#define BAUDRATE_VAL_28800					68
#define BAUDRATE_VAL_38400					51
#define BAUDRATE_VAL_57600					34
#define BAUDRATE_VAL_76800					25
#define BAUDRATE_VAL_115200					16
#endif

#ifdef _SYS_CLK_AT147456MHZ
#define BAUDRATE_VAL_9600					95
#define BAUDRATE_VAL_19200					47
#define BAUDRATE_VAL_28800					31
#define BAUDRATE_VAL_38400					24
#define BAUDRATE_VAL_57600					15
#define BAUDRATE_VAL_76800					11
#define BAUDRATE_VAL_115200					7
#endif

#define BAUDRATE_VAL_DEFAULT				BAUDRATE_VAL_57600

#define BAUD_RATE(xtal,baud,smod)       	(256L-xtal/(baud*smod*192L))
//#define T2RELOAD(Baudrate)   				(65536-(XTAL/32/(Baudrate)))
#define T2RELOAD(Baudrate)   				(65536-((XTAL/2)/32/(Baudrate)))

#define SERIAL_MODE1                    	0x40  // SCON.6 and SCON.7
#define DOUBLE_BAUD_RATE                	0x80  // PCON.7
#define ENABLE_RECEIVE                  	0x10  // SCON.4

#define UART_RXINT_ENABLE           		0x80
#define UART_TXINT_ENABLE           		0x40
#define UART_RX_ENABLE              		0x10
#define UART_TX_ENABLE              		0x08

#define UART_RX_COMPLETE            		0x80
#define UART_TX_COMPLETE            		0x40
#define UART_DATA_EMPTY             		0x20

#define TXD0_SEL_EN							BIT1
#define RXD0_SEL_EN							BIT0
#define TXD1_SEL_EN							0x40
#define RXD1_SEL_EN							0x80

#define UART_DB_BAUDRATE					0x02


#ifdef _UART0_ENABLE
//------------------------------------------------------
//Atmega128 Serial Register
#ifdef _HAL_MEGA128_UART
#define U0R_DATA							UDR0
#define U0R_CSRA							UCSR0A
#define U0R_CSRB							UCSR0B
#define U0R_UBRR							UBRR0L
#endif
//------------------------------------------------------
//Atmega8 Serial Register
#ifdef _HAL_MEGA8_UART
#define U0R_DATA							UDR
#define U0R_CSRA							UCSRA
#define U0R_CSRB							UCSRB
#define U0R_UBRR							UBRRL
#endif
//------------------------------------------------------

//------------------------------------------------------------------------------
// Serial Prot0 Utility Fuction Routine
#define u0_WaitForReceive()					while(!(UCSR0A & UART_RX_COMPLETE))
#define u0_RxData()		 					(U0R_DATA)

#define _u0_TxData(Data)					(U0R_DATA = Data)
#define u0_WaitForSend()					while(!(U0R_CSRA & UART_DATA_EMPTY))

#define u0_IsReceiveData()					((U0R_CSRA & UART_RX_COMPLETE))

#define u0_EnableRxIsr()					(U0R_CSRB |= (UART_RXINT_ENABLE|UART_RX_ENABLE)) 
#define u0_EnableRx()						(U0R_CSRB |= UART_RX_ENABLE)
#define u0_EnableTx()						(U0R_CSRB |= UART_TX_ENABLE)
	
void u0_baudrate(unsigned char baud);
void u0_hal_init(void);

void u0_TxData(unsigned char Data)
{
	u0_WaitForSend();
	_u0_TxData(Data);
}

void u0_hal_init(void)
{
	// enable serial uart & receiver
	u0_EnableTx();
	u0_EnableRx(); 
	
	//at 16Mhz
	U0R_CSRA |= UART_DB_BAUDRATE;
	       
  	fdevopen(u0_TxData,  0);
}

void u0_baudrate(unsigned char baud)
{
	unsigned char baud_rate = 0;
	
	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	
	
    U0R_UBRR = baud_rate;
}
//------------------------------------------------------------------------------
#endif	//end _UART0_ENABLE


#ifdef _UART1_ENABLE
//------------------------------------------------------------------------------
// Serial Prot1 Utility Fuction Routine
#define U1_printf							printf

#define u1_WaitForReceive()					while(!(UCSR1A & UART_RX_COMPLETE))
#define u1_RxData()		 					(UDR1)

#define _u1_TxData(Data)					(UDR1 = Data)
#define u1_WaitForSend()					while(!(UCSR1A & UART_DATA_EMPTY))

#define u1_IsReceiveData()					((UCSR1A & UART_RX_COMPLETE))

#define u1_EnableRxIsr()					(UCSR1B |= (UART_RXINT_ENABLE|UART_RX_ENABLE)) 
	
void u1_baudrate(unsigned char baud);


void u1_TxData(unsigned char Data)
{
	u1_WaitForSend();
	_u1_TxData(Data);
}



void u1_hal_init(void)
{
	// enable serial uart & receiver
    UCSR1B = UART_TX_ENABLE|UART_RX_ENABLE;	 
	
	//at 16Mhz
	UCSR1A |= UART_DB_BAUDRATE;

  	fdevopen(u1_TxData,  0);
}

void u1_baudrate(unsigned char baud)
{
	switch (baud)
	{
		case BAUD_9600:
			UBRR1L = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			UBRR1L = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			UBRR1L = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			UBRR1L = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			UBRR1L = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			UBRR1L = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			UBRR1L = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			UBRR1L = BAUDRATE_VAL_115200;
			break;		
		default :
			UBRR1L = BAUDRATE_VAL_DEFAULT;			
			break;
	}	
}
//------------------------------------------------------------------------------
#endif