/**
 @file		evb.c
 @brief 		functions to initialize EVB prepheral equipments
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "../mcu/types.h"
#include "../iinchip/socket.h"



#include "w5100.h"
#include "wiznet.h"

#include "../../../00_hal/system.h"
#include "../../spi.h"

        
#define	ATMEGA128_0WAIT		0
#define	ATMEGA128_1WAIT		1
#define	ATMEGA128_2WAIT		2
#define	ATMEGA128_3WAIT		3
#define ATMEGA128_NUM_WAIT	ATMEGA128_1WAIT


void InitWiznet(void) 
{
	cli();
		
#ifndef __DEF_IINCHIP_INT__	
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;
	EIFR=0x00;
#else
	EICRA = 0x00;			// External Interrupt Control Register A clear
	EICRB = 0x02;			// External Interrupt Control Register B clear // edge 
	EIMSK = (1 << INT4);		// External Interrupt Mask Register : 0x10
	EIFR = 0xFF;			// External Interrupt Flag Register all clear
	DDRE &= ~(1 << INT4);		// Set PE Direction 
	PORTE |= (1 << INT4);		// Set PE Default value
#endif
 

#if (ATMEGA128_NUM_WAIT == ATMEGA128_0WAIT)
	MCUCR = 0x80;		
	XMCRA=0x40;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_1WAIT)
	MCUCR = 0xc0;		// MCU control regiseter : enable external ram
	XMCRA=0x40;		// External Memory Control Register A : 
						// Low sector   : 0x1100 ~ 0x7FFF
						// Upper sector : 0x8000 ~ 0xFFFF
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_2WAIT )
	MCUCR = 0x80;
	XMCRA=0x42;
#elif ((ATMEGA128_NUM_WAIT == ATMEGA128_3WAIT)
	MCUCR = 0xc0;
	XMCRA=0x42;
#else
#error "unknown atmega128 number wait type"
#endif	


	#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_SPI_MODE__)
		//SPI Initialize
	IINCHIP_SpiInit();

	IINCHIP_CSInit();
	IINCHIP_CSon();
	#endif  
	
	//sei();				// enable interrupts
}




void DisplayConfig(void)
{
	u_char addr[6];
	u_char i = 0;
	u_long iaddr;
	printf("\r\n================================================\r\n");
	printf("       Net Config Information\r\n");
	printf("================================================\r\n");
	

	for(i=0; i<6;i++)addr[i] = IINCHIP_READ(SHAR0+i);
	printf("MAC ADDRESS      : 0x%02X.0x%02X.0x%02X.0x%02X.0x%02X.0x%02X\r\n",addr[0],addr[1],addr[2],addr[3],addr[4],addr[5]);
	

	printf("SUBNET MASK      : ");
	for(i=0; i < 4; i++)
	{
		printf("%d.", (char)IINCHIP_READ(SUBR0+i));
	}


	printf("\r\nG/W IP ADDRESS   : ");
	for(i=0; i < 4; i++)
	{
		printf("%d.", (char)IINCHIP_READ(GAR0+i));
	}

	printf("\r\nLOCAL IP ADDRESS : ");
	for(i=0; i < 4; i++)
	{
		printf("%d.", (char)IINCHIP_READ(SIPR0+i));
	}

	printf("\r\n================================================\r\n");		
}


/**
@brief	ntohs function converts a unsigned short from TCP/IP network byte order to host byte order (which is little-endian on Intel processors).
@return 	a 16-bit number in host byte order
*/

u_long swapl(u_long l)
{
	u_long ret=0;
	ret = (l & 0xFF) << 24;
	ret |= ((l >> 8) & 0xFF) << 16;
	ret |= ((l >> 16) & 0xFF) << 8;
	ret |= ((l >> 24) & 0xFF);
	return ret;
}


unsigned long htonl(unsigned long hostlong)
{
#if ( SYSTEM_ENDIAN == _ENDIAN_LITTLE_ )
	return swapl(hostlong);
#else
	return hostlong;
#endif	
}

